//#include "general.h"
#include "btrie.h"

/* Inserts a key for a word in a trie. If the word given as input is EPSILON, or the trie already contains that key, it returns 1. Otherwise, it returns 0. */
int insertTrie(struct trieNode *root, char *word, int key)
{
  struct trieNode *current;
  int i;

  if (key == 0)
     return 1;
  /*-
  if (strcmp(word,"0")==0)
{     printf("%s\n", word);
     exit(1);}
     //strcat(word, "aha");
     */

  current = &root[(int)word[0]-FIRSTVALIDCHAR];
  for (i = 1; i < strlen(word); i++) {
    if (!current->link) {
      current->link = (struct trieNode *) calloc(VALIDCHARRANGE, sizeof(struct trieNode));
      if (!current->link) {
        printf/*addWarningMessage*/("Memory allocation failed for a trie node!\n");
        exit(1);
        //terminateProgram(INSUFFICENT_MEMORY);
      }
    }
    current = &(current->link[(int)word[i]-FIRSTVALIDCHAR]);
  }
  if (current->key)
    return 1;
  current->key = key;
  return 0;
}

/* Searchs for a word in a trie. Returns the key found in the variable result. If the key is not found, function return 0. In that case, content of result is not important. */
int searchTrie(struct trieNode *root, char *word, int *result)
{
  struct trieNode *current;
  int i;
  /*
  if (!strcmp(word, "0")) {
    *result = 0;
    return 1;
  }
  */
  current = &root[(int)word[0]-FIRSTVALIDCHAR];
  for (i = 1; i < strlen(word); i++) {
    if (!current->link)
      return 0;
    current = &(current->link[(int)word[i]-FIRSTVALIDCHAR]);
  }
  if (!current->key)
    return 0;
  *result = current->key;

  return 1;
}

/* Checks whether a level of a trie is filled. */
int trieFilled(struct trieNode *root)
{
  int i;

  for (i = FIRSTVALIDCHAR; i <= LASTVALIDCHAR; i++)
    if (root[i-FIRSTVALIDCHAR].link || root[i-FIRSTVALIDCHAR].key)
      return 1;
  return 0;
}

/* Prints the trie to screen. It is a recursive function. Hence, a null input string is needed when calling this function. */
void printTrieToScreen(struct trieNode *root, char *word)
{
  struct trieNode *current;
  int i, length;

  for (i = FIRSTVALIDCHAR; i <= LASTVALIDCHAR; i++) {
    current = &root[i-FIRSTVALIDCHAR];
    if (current->link || current->key) {
      length = strlen(word);
      word[length] = (char)i;
      word[length+1] = '\0';
      if (current->key)
        printf("%20s %3d\n", word, current->key);
      if (current->link)
        printTrieToScreen(current->link, word);
      else {
        length = strlen(word);
        word[length-1] = '\0';
      }
    }
  }
  length = strlen(word);
  if (length > 0)
    word[length-1] = '\0';
}

/* Allocates space for the root of a trie. */
void allocateTrie(struct trieNode **root)
{
  *root = (struct trieNode *) calloc(VALIDCHARRANGE, sizeof(struct trieNode));
}

/* Deallocates a trie with a given root. */
void deallocateTrie(struct trieNode *root)
{
  struct trieNode *current;
  int i;

  for (i = FIRSTVALIDCHAR; i <= LASTVALIDCHAR; i++) {
    current = &root[i-FIRSTVALIDCHAR];
    if (current->link) {
      deallocateTrie(current->link);
    }
  }
  free(root);
}
